@@ -1151,7 +1151,9 @@ static bool try_groups(Conf *conf, Status *status, const char *msg_path, int fd)
             goto fail;
         }
         else if (pid == 0) {
-            // Child / sendmail process.
+            // Child / sendmail process. Any errors in this branch should cause
+            // the child process to exit rather than try to continue executing
+            // as if it was the parent process.
 
             close(STDOUT_FILENO);
             if (dup2(pipeto[0], STDIN_FILENO) == -1 || dup2(pipefrom[1],
@@ -1161,7 +1163,7 @@ static bool try_groups(Conf *conf, Status *status, const char *msg_path, int fd)
                 close(pipefrom[0]);
                 close(pipefrom[1]);
                 syslog(LOG_CRIT, "try_groups: dup2: %s", strerror (errno));
-                goto fail;
+                exit(1);
             }
             close(pipeto[0]);
             close(pipeto[1]);
@@ -1184,7 +1186,7 @@ static bool try_groups(Conf *conf, Status *status, const char *msg_path, int fd)
             execvp(cur_ext->sendmail_argv[0], (char **const) sub_argv);
             free(sub_argv);
             syslog(LOG_CRIT, "try_groups: execvp: %s", strerror (errno));
-            goto fail;
+            exit(1);
         }
         else {
             // Parent process.
@@ -1207,9 +1209,15 @@ static bool try_groups(Conf *conf, Status *status, const char *msg_path, int fd)
             if (waitpid(pid, &rtn_status, 0) || WIFEXITED(rtn_status)) {
                 int child_rtn = WEXITSTATUS(rtn_status);
                 if (child_rtn != 0) {
-                    syslog(LOG_ERR, "%s: Received error %d when executing "
-                      "'%s' on '%s': %.*s", cur_ext->name, child_rtn,
-                      cur_ext->sendmail, msg_path, (int) stderrbuf_used, stderrbuf);
+                    if (stderrbuf_used == 0) {
+                        syslog(LOG_ERR, "%s: Received error %d when executing "
+                          "'%s' on '%s'", cur_ext->name, child_rtn,
+                          cur_ext->sendmail, msg_path);
+                    } else {
+                        syslog(LOG_ERR, "%s: Received error %d when executing "
+                          "'%s' on '%s': %.*s", cur_ext->name, child_rtn,
+                          cur_ext->sendmail, msg_path, (int) stderrbuf_used, stderrbuf);
+                    }
                     goto next;
                 }
             }
@@ -1491,16 +1499,27 @@ static void check_externals(const char *file)
 
 static void usage(int rtn_code)
 {
-    fprintf(stderr, "Usage: %s [-m <batch|daemon>] [-t <conf>]\n", __progname);
+    fprintf(stderr, "Usage: %s [-hv] [-m <batch|daemon>] [-t <conf>]\n", __progname);
     exit(rtn_code);
 }
 
 
+static void version()
+{
+  puts(PACKAGE_NAME " " PACKAGE_VERSION);
+}
+
+
 int main(int argc, char** argv)
 {
+#ifdef HAVE_PLEDGE
+    if (pledge("stdio rpath wpath cpath flock getpw proc exec", NULL) == -1)
+        err(EXIT_FAILURE, "pledge");
+#endif
+
     Mode mode = BATCH_MODE;
     int ch;
-    while ((ch = getopt(argc, argv, "hm:t:")) != -1) {
+    while ((ch = getopt(argc, argv, "hm:t:v")) != -1) {
         switch (ch) {
             case 'm':
                 if (strcmp(optarg, "batch") == 0)
@@ -1517,6 +1536,10 @@ int main(int argc, char** argv)
                 check_externals(optarg);
                 exit(0);
                 break;
+            case 'v':
+                version();
+                exit(0);
+                break;
             default:
                 usage(1);
         }
